package com.localtransfer;

import android.animation.ObjectAnimator;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ConcurrentModificationException;
import java.util.List;

public class ServerFile extends Transfer {

    private Integer id;

    public String name;

    public String href;

    public String mime;

    public String type;

    public long size;

    public String save_location;

    public Uri uri;

    public Button button;

    private static List<ServerFile> instances = new ArrayList<>();

    public ServerFile(Integer id) {
        this.id = id;
        instances.add(this);
        drawable = R.drawable.ic_download_24;
        info = "Download complete";
        state = STATE_NOT_REQUESTED;
    }

    @Override
    public void StartTransfer() {
        super.StartTransfer();

        if (uri == null && android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.Q) {
            ContentValues values = new ContentValues();
            values.put(MediaStore.MediaColumns.DISPLAY_NAME, name);
            values.put(MediaStore.MediaColumns.MIME_TYPE, mime);
            values.put(MediaStore.MediaColumns.RELATIVE_PATH, Transfer.local_storage);
            uri = resolver.insert(MediaStore.Downloads.EXTERNAL_CONTENT_URI, values);
        }
        try {
            downloadFile();
        } catch (IOException e) {
            final String ExceptionName = e.getClass().getSimpleName();
            final String ExceptionMess = e.getMessage();

            state = Transfer.STATE_FAILED;
            error = ExceptionName + ": " + ExceptionMess;
            PostProgress();

            if(ExceptionName != null && ExceptionMess != null) {
                errorSnackbar(ExceptionMess);
            }

        }

        try {
            for (Object obj : getInstances()) {
                Transfer transfer = (Transfer) obj;
                if (transfer.state == Transfer.STATE_STANDBY)
                    StartTransfer();
            }
        } catch (ConcurrentModificationException e) {
            e.printStackTrace();
            try {
                Thread.sleep(10);
            } catch (InterruptedException InterrupEx) {
                InterrupEx.printStackTrace();
            }
            for (Object obj : getInstances()) {
                Transfer transfer = (Transfer) obj;
                if (transfer.state == Transfer.STATE_STANDBY)
                    StartTransfer();
            }
        }

        /*Intent serviceIntent = new Intent(Transfer.activity, TransferService.class);
        Transfer.activity.stopService(serviceIntent);
        Transfer.runningTransfer = false;*/
    }

    public static List getInstances() {
        return instances;
    }

    public static ServerFile getFileById(int id) {
        for (Object obj: instances) {
            ServerFile p = (ServerFile) obj;
            if (p.id == id) return p;
        }
        return null;
    }

    public Integer getId() {
        return id;
    }

    public boolean exist(Context context) {
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.Q) {
            Cursor cursor = context.getContentResolver().query(MediaStore.Downloads.EXTERNAL_CONTENT_URI, null, null, null, null);
            while (cursor.moveToNext()) {
                String MediaStore_File_name = cursor.getString(cursor.getColumnIndexOrThrow(MediaStore.Downloads.DISPLAY_NAME));
                long MediaStore_File_size = cursor.getLong(cursor.getColumnIndexOrThrow(MediaStore.Downloads.SIZE));
                if(MediaStore_File_name.equals(name) && MediaStore_File_size == size) {
                    int cursor_id = cursor.getInt(cursor.getColumnIndexOrThrow(MediaStore.Downloads._ID));
                    uri = ContentUris.withAppendedId(MediaStore.Downloads.EXTERNAL_CONTENT_URI, cursor_id);
                    return true;
                }
            }
        }
        else {
            File file = new File(save_location, name);
            new File(save_location).mkdirs();
            uri = Uri.fromFile(file);
            if (file.exists() && file.length() == size)
                return true;
        }
        return false;
    }

    public boolean setButton(Button button) {
        if (this.state == STATE_RUNNING) {
            this.button = button;
            return true;
        }
        return false;
    }

    public InputStream getThumbnail() {
        InputStream thumbnail = null;
        URL url;
        try {
            String query = URLEncoder.encode(href, StandardCharsets.UTF_8.toString());
            String serverUrl = root + "/thumbnail.php?file=" + query;
            url = new URL(protocol, host, port, serverUrl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("GET");
            conn.setConnectTimeout(2000);

            conn.connect();

            thumbnail = conn.getInputStream();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return thumbnail;
    }

    public void downloadFile() throws IOException {

        URL url = null;
        HttpURLConnection conn = null;
        int maxBufferSize = 1 * 1024 * 1024;
        byte[] buffer = new byte[maxBufferSize];
        int bufferLength;
        loaded = 0;

        PreProgress();

        String[] parts = href.split("/");
        String path = "";
        for(int i=0; i< parts.length - 1; i++) {
            path = path + parts[i] + "/";
        }
        String query = URLEncoder.encode(parts[parts.length - 1], StandardCharsets.UTF_8.toString());
        url = new URL(protocol, host, port, path + query.replace("+", "%20"));
        conn = (HttpURLConnection) url.openConnection();
        conn.setDoOutput(true);
        conn.setRequestMethod("GET");
        conn.setConnectTimeout(2000);

        Log.d("URL", url.toString());

        conn.connect();

        int responseCode = conn.getResponseCode();

        if (responseCode == HttpURLConnection.HTTP_OK) {

            OutputStream fileOutput = activity.getContentResolver().openOutputStream(uri);
            InputStream in = conn.getInputStream();

            while ((bufferLength = in.read(buffer)) > 0) {
                fileOutput.write(buffer, 0, bufferLength);
                loaded+= bufferLength;
                percent = ((loaded * 100) / size);
                ProgressUpdateDelay();
            }
            fileOutput.close();

            conn.disconnect();

            message = "File " + name + " successful download";
            state = Transfer.STATE_SUCCESS;
            PostProgress();

        }
        else {
            String s = conn.getResponseMessage();
            throw new HttpErrorException("error code: " + responseCode + " " + s);
        }

        conn.disconnect();
    }

    public String deleteFile() throws IOException {

        URL url = null;
        HttpURLConnection conn = null;

        String message;

        String query = URLEncoder.encode(name, StandardCharsets.UTF_8.toString());
        String serverUrl = root + "/delete.php?file=" + query;
        url = new URL(protocol, host, port, serverUrl);
        conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");
        conn.setConnectTimeout(2000);

        conn.connect();

        int responseCode = conn.getResponseCode();

        if (responseCode == HttpURLConnection.HTTP_OK) {

            message = "File " + name + " successful deleted";
        }
        else {
            String s = conn.getResponseMessage();
            throw new HttpErrorException("error code: " + responseCode + " " + s);
        }

        conn.disconnect();

        return message;
    }

    public void PreProgress() {
        activity.runOnUiThread(() -> {

            final Drawable image = Transfer.activity.getDrawable(R.drawable.ic_spinner_rotate);
            sizeSI = Transfer.humanReadableByteCountBin(size);

            state = STATE_RUNNING;

            if(button != null) {
                int h = image.getIntrinsicHeight();
                int w = image.getIntrinsicWidth();
                image.setBounds(0, 0, w, h);
                button.setCompoundDrawables(button.getCompoundDrawables()[0], null, image, null);
                ObjectAnimator anim = ObjectAnimator.ofInt(image, "level", 0, 10000);
                anim.setDuration(1000);
                anim.setRepeatCount(Animation.INFINITE);
                anim.setInterpolator(new LinearInterpolator());
                anim.start();
            }
        });
    }

    public void ProgressUpdateDelay() {
        // Run only every second to reduce CPU usage
        if (System.currentTimeMillis() > (savedTimeMillis + 1000)) {
            savedTimeMillis = System.currentTimeMillis();
            ProgressUpdate();
        }
    }

    private void ProgressUpdate(){

        loadedSI = Transfer.humanReadableByteCountBin(loaded);

        activity.runOnUiThread(() -> {

            if (app_started && button != null)
                button.setText(String.format("Download in progress %d%%", percent));

            try {

                notifBuilder.setSmallIcon(R.drawable.ic_upload_and_download_from_the_cloud)
                        .setContentTitle(name)
                        .setContentText(String.format("%d%% %s/%s", percent, loadedSI, sizeSI))
                        .setProgress(100, (int) percent, false)
                        .setContentIntent(pendingIntent);
                notifiManager.notify(Transfer.NOTIF_SERVICE, notifBuilder.build());

                if (app_started && fragment_on)
                    showProgressFragment();

            } catch (ConcurrentModificationException e) {
                e.printStackTrace();
            }
        });
    }

    public void PostProgress() {
        ProgressUpdate();
        activity.runOnUiThread(() -> {

            notifiManager.cancel(Transfer.NOTIF_SERVICE);

            if (!app_started) {
                notifBuilder.setSmallIcon(R.drawable.ic_upload_and_download_from_the_cloud)
                        .setContentTitle(name)
                        .setContentText(info)
                        .setProgress(0, 0, false)
                        .setContentIntent(pendingIntent);
                notifiManager.notify(id, notifBuilder.build());
            }

            if (button != null) {
                final LinearLayout layout = (LinearLayout) button.getParent();
                layout.findViewById(R.id.file_view).setVisibility(LinearLayout.VISIBLE);
                layout.findViewById(R.id.file_share).setVisibility(LinearLayout.VISIBLE);
                button.setVisibility(LinearLayout.GONE);
                button.setEnabled(true);

                button.setText(activity.getString(R.string.file_download));
                button.setCompoundDrawables(button.getCompoundDrawables()[0], null, null, null);
            }

            System.out.println(message);
            /*Snackbar.make(Transfer.activity.findViewById(R.id.view_pager), tr.message, Snackbar.LENGTH_LONG)
                    .setAction("Action", null).show();*/
            Toast.makeText(activity, message, Toast.LENGTH_SHORT).show();

        });
    }

    public void showProgressFragment(){

        if(Arrays.asList(STATE_RUNNING, STATE_SUCCESS, STATE_FAILED).contains(state)) {

            LinearLayout groot = fragment_progress.findViewById(R.id.groot);

            View progress = groot.findViewById(id);

            if (progress == null) {
                progress = inflater.inflate(R.layout.progress, null);
                progress.setId(id);
                final View final_progress = progress;
                activity.runOnUiThread(() -> {
                    groot.addView(final_progress, 0);
                });
            }

            ((ProgressBar) progress.findViewById(R.id.progressBar)).setProgress((int) percent);
            ((ImageView) progress.findViewById(R.id.transferType)).setImageResource(drawable);
            ((TextView) progress.findViewById(R.id.progressText)).setText(percent + " %");
            ((TextView) progress.findViewById(R.id.fileName)).setText(name);
            ((TextView) progress.findViewById(R.id.fileSize)).setText(String.format("%s/%s", loadedSI, sizeSI));

            if (state == Transfer.STATE_SUCCESS)
                ((TextView) progress.findViewById(R.id.fileName)).setTextColor(Color.GREEN);
            else if (state == Transfer.STATE_FAILED)
                ((TextView) progress.findViewById(R.id.fileName)).setTextColor(Color.RED);

        }
    }
}
